from django import forms
from acclaim_badges.models import AcclaimToken
from acclaim_badges.models import BadgeCourse
from acclaim_badges.models import AcclaimApi
from openedx.core.djangoapps.content.course_overviews.models import CourseOverview

class BadgeCourseForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(BadgeCourseForm, self).__init__(*args, **kwargs)
        acclaim_api = AcclaimApi()
        template_list = acclaim_api.template_choices()
	badge_default = [('','Select Badge')]
	course_default = [('','Select Course')]

        self.fields['badge_template'] = forms.ChoiceField(
		choices= (badge_default + template_list),
	)

	make_tuple = lambda x:(str(x.id), x.display_name)
	courses = CourseOverview.objects.all()
        self.fields['edx_course'] = forms.ChoiceField(
            choices=(course_default + map(make_tuple, courses)),
	)

    class Meta:
        model = BadgeCourse
        fields = ['badge_template', 'edx_course']

class AcclaimTokenForm(forms.ModelForm):
    auth_token = forms.CharField(widget=forms.PasswordInput())

    class Meta:
        model = AcclaimToken
        fields = ['auth_token', 'organization_id', 'url']



# -*- coding: utf-8 -*-
"""
URLs for acclaim_badges.
"""
from __future__ import absolute_import, unicode_literals
from django.contrib.auth.decorators import login_required

from django.conf.urls import url
from django.views.generic import TemplateView
# from lms.djangoapps.acclaim_badges import views
from .views import BadgeCourseList
from .views import BadgeCourseCreate
from .views import BadgeCourseDelete
from .views import BadgeCourseUpdate

from .views import AcclaimTokenList
from .views import AcclaimTokenCreate
from .views import AcclaimTokenDelete
from .views import AcclaimTokenUpdate

urlpatterns = [
    url(r'^tokens/', AcclaimTokenList.as_view(), name='acclaim-tokens'),
    url(r'^token/add/$', AcclaimTokenCreate.as_view(), name='acclaim-token-add'),
    url(r'^token/update/(?P<pk>[0-9]+)/$', AcclaimTokenUpdate.as_view(), name='acclaim-token-update'),
    url(r'^token/delete/(?P<pk>[0-9]+)/$', AcclaimTokenDelete.as_view(), name='acclaim-token-delete'),

    url(r'badge-course/add/$', BadgeCourseCreate.as_view(), name='badge-course-add'),
    url(r'^badge-courses/$', BadgeCourseList.as_view(), name='badge-courses'),
    url(r'^badge-course/delete/(?P<pk>[0-9]+)/$', BadgeCourseDelete.as_view(), name='badge-course-delete'),
    url(r'^badge-course/update/(?P<pk>[0-9]+)/$', BadgeCourseUpdate.as_view(), name='badge-course-update'),
]



"""
Acclaim Badges for EDX.
"""

from __future__ import absolute_import, unicode_literals

__version__ = '0.1.0'

default_app_config = 'acclaim_badges.apps.AcclaimBadgesConfig'  # pylint: disable=invalid-name



# -*- coding: utf-8 -*-
"""
Database models for acclaim_badges.
"""

from __future__ import absolute_import, unicode_literals

from django.db import models
from django.dispatch import receiver
from openedx.core.djangoapps.signals.signals import COURSE_GRADE_CHANGED
from django.contrib.auth.models import User
import requests
import json
import datetime
from django.core.urlresolvers import reverse

from django.utils.encoding import python_2_unicode_compatible
from model_utils.models import TimeStampedModel
from openedx.core.djangoapps.content.course_overviews.models import CourseOverview

from opaque_keys.edx.keys import CourseKey, UsageKey
from openedx.core.djangoapps.content.course_overviews.models import CourseOverview
from django.core.exceptions import ObjectDoesNotExist

from encrypted_fields import EncryptedTextField

from logging import getLogger
log = getLogger(__name__)

@receiver(COURSE_GRADE_CHANGED)
def my_callback(sender, user, course_grade, course_key, deadline, **kwargs):
    log.info('Grade change trigger recieved.  Checking to see if earner is qualified for badging.') 
    for usr in User.objects.filter(username=user):
	if course_grade.summary['grade'] == 'Pass':
            log.info('Earner earned a passing score.')
            courses = BadgeCourse.objects.filter(edx_course=str(course_key))
            if courses.exists():
                acclaim_api = AcclaimApi()
                for course in courses:
                    badge_template_id = json.loads(course.badge_template)[0]
                    acclaim_api.issue_badge(usr, badge_template_id)

class SingletonModel(models.Model):
    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        self.__class__.objects.exclude(id=self.id).delete()
        super(SingletonModel, self).save(*args, **kwargs)

    @classmethod
    def load(cls):
        try:
            return cls.objects.get()
        except cls.DoesNotExist:
            return cls()

class AcclaimToken(SingletonModel, TimeStampedModel):
    auth_token = EncryptedTextField()
    organization_id = models.UUIDField()

    PROD = 'youracclaim.com [Production]'
    SANDBOX = 'jefferson-sandbox.herokuapp.com [Sandbox]'
    
    ENVIRONMENTS = (
        ('https://jefferson-sandbox.herokuapp.com/', SANDBOX),
        ('https://youracclaim.com/', PROD),
    )

    url = models.CharField(
        max_length=200,
        choices=ENVIRONMENTS,
    )

    def get_absolute_url(self):
        return reverse('acclaim-tokens')

class AcclaimApi():
    def __init__(self):
        abstract = True

    def get_token(self):
        token = AcclaimToken.objects.all()
        if token.exists():
            return AcclaimToken.objects.all()[0]
        else:
            return False

    def parse_acclaim_response(self, response):
        templates = []
        if response is not None:
            if response.status_code == 200:
                templates = json.loads(response.text)['data'] 
        return templates

    def is_badge_duplicate(self, badge_template_id, recipient_email):
        log.info('Checking badge duplicate email: {}, template: {}'.format(recipient_email, badge_template_id ))
        duplicate = False
        params = "?filter=query::{}|badge_template_id::{}".format(recipient_email, badge_template_id)
        token = self.get_token()
        if token:
            url = "{}/v1/organizations/{}/badges".format(str(token.url), str(token.organization_id))
            query = url + params
            try:
                response = requests.get(query, auth=(token.auth_token, ''))
                json = response.json()
                existing_badge_count = json['metadata']['count']
                log.info('user badge count: {}'.format(existing_badge_count))
                if existing_badge_count > 0:
                    log.info('user already has a badge with the given template ID and email')
                    duplicate = True
                else:
                    log.info('No badge yet exists for earner.')
            except (ConnectionError, requests.exceptions.MissingSchema) as e:
                log.warning('Unable to connect to Acclaim')
        return duplicate

    def get_templates(self):
        from requests.exceptions import ConnectionError
        templates = []
        token = self.get_token()

        if token:
            url = "{}v1/organizations/{}/badge_templates".format(str(token.url), str(token.organization_id))
            try:
                response = requests.get(url, auth=(token.auth_token, ''))
                templates = self.parse_acclaim_response(response)
            except (ConnectionError, requests.exceptions.MissingSchema) as e:
                log.warning('Unable to connect to Acclaim')
        return templates

    def issue_badge(self, usr, template_id):
        log.info('Issue Acclaim Badge with template: {}.'.format(template_id))
        if not self.is_badge_duplicate(template_id, usr.email):
            token = self.get_token()
            if token:
                date = str(datetime.datetime.now().strftime('%Y-%m-%dT%H:%M:%S')) + ' Z'
                payload = {'recipient_email': usr.email, 'badge_template_id': template_id, 'issued_at': date}
                url = "{}/v1/organizations/{}/badges".format(str(token.url), str(token.organization_id))
                r = requests.post(url, data = payload, auth=(token.auth_token, ''))
                log.info('Acclaim badge payload: ' + str(payload))
                log.info('Acclaim badge issue: {}'.format(str(r)))
                if r.status_code != 201:
                    log.warning('Unable to connect to Acclaim to issue badge')

    def template_choices(self):
        templates = self.get_templates()
        make_tuple = lambda x:( json.dumps([x['id'], x['name']]) , x['name'])
        return map(make_tuple, templates)

class BadgeCourse(models.Model):
    @property
    def course_name(self):
        course_display_name = "Does not exist"
        try:
            course_obj = CourseOverview.objects.get(pk=CourseKey.from_string(self.edx_course))
            course_display_name = course_obj.display_name
        except ObjectDoesNotExist:
            log.warning("Course:{} does not exist".format(str(self.edx_course)))
        return course_display_name

    badge_template = models.CharField(
        max_length=200,
    )

    edx_course = models.CharField(
        max_length=200,
    )

    def get_absolute_url(self):
        return reverse('badge-courses')



from django.shortcuts import get_object_or_404, render
from django.http import HttpResponseRedirect, HttpResponse
from django.core.urlresolvers import reverse, reverse_lazy
from django.http import JsonResponse

from django.utils import timezone
from django.views.generic import ListView
from django.views.generic.detail import DetailView
from django.views.generic.edit import CreateView
from django.views.generic.edit import DeleteView
from django.views.generic.edit import UpdateView

from acclaim_badges.models import BadgeCourse
from acclaim_badges.models import AcclaimToken
from django.contrib.auth.decorators import user_passes_test
from django.forms import ModelForm
from .forms import AcclaimTokenForm
from .forms import BadgeCourseForm

from django.contrib.auth.decorators import login_required
from django.contrib.auth.decorators import user_passes_test
from django.utils.decorators import method_decorator

# badge course list controllers
class BadgeCourseDelete(DeleteView):
    model = BadgeCourse
    success_url = reverse_lazy('badge-courses')

    @method_decorator(user_passes_test(lambda u: u.is_superuser, login_url='/admin/login'))
    def dispatch(self, *args, **kwargs):
            return super(DeleteView, self).dispatch(*args, **kwargs)

class BadgeCourseList(ListView):
    model = BadgeCourse

    @method_decorator(user_passes_test(lambda u: u.is_superuser, login_url='/admin/login'))
    def dispatch(self, *args, **kwargs):
            return super(ListView, self).dispatch(*args, **kwargs)

class BadgeCourseCreate(CreateView):
    model = BadgeCourse
    form_class = BadgeCourseForm

    @method_decorator(user_passes_test(lambda u: u.is_superuser, login_url='/admin/login'))
    def dispatch(self, *args, **kwargs):
            return super(CreateView, self).dispatch(*args, **kwargs)

class BadgeCourseUpdate(UpdateView):
    model = BadgeCourse
    form_class = BadgeCourseForm
    success_url = reverse_lazy('badge-courses')

    @method_decorator(user_passes_test(lambda u: u.is_superuser, login_url='/admin/login'))
    def dispatch(self, *args, **kwargs):
            return super(UpdateView, self).dispatch(*args, **kwargs)

# acclaim token controllers
class AcclaimTokenList(ListView):
    model = AcclaimToken

    @method_decorator(user_passes_test(lambda u: u.is_superuser, login_url='/admin/login'))
    def dispatch(self, *args, **kwargs):
            return super(ListView, self).dispatch(*args, **kwargs)

class AcclaimTokenCreate(CreateView):
    model = AcclaimToken
    fields = ['auth_token', 'organization_id', 'url']

    @method_decorator(user_passes_test(lambda u: u.is_superuser, login_url='/admin/login'))
    def dispatch(self, *args, **kwargs):
            return super(CreateView, self).dispatch(*args, **kwargs)

class AcclaimTokenDelete(DeleteView):
    model = AcclaimToken
    success_url = reverse_lazy('acclaim-tokens')

    @method_decorator(user_passes_test(lambda u: u.is_superuser, login_url='/admin/login'))
    def dispatch(self, *args, **kwargs):
            return super(DeleteView, self).dispatch(*args, **kwargs)

class AcclaimTokenUpdate(UpdateView):
    model = AcclaimToken
    form_class = AcclaimTokenForm
    success_url = reverse_lazy('acclaim-tokens')

    @method_decorator(user_passes_test(lambda u: u.is_superuser, login_url='/admin/login'))
    def dispatch(self, *args, **kwargs):
            return super(UpdateView, self).dispatch(*args, **kwargs)
